"use strict";

exports.__esModule = true;
exports.default = void 0;

var _util = require("./util");

var _default = (t, refs) => {
  const hoistTargets = [];
  refs.forEach(referencePath => {
    var _referencePath, _callee, _tail;

    const caller = (0, _util.findTargetExpression)(referencePath);

    if (!caller) {
      (0, _util.throwFrameError)(referencePath, 'Placeholders must be used as function arguments or the\n' + 'right side of a variable declaration, ie. `const eq = _ === _`');
    }

    const callee = (0, _util.findTargetCallee)(referencePath);
    const wrapper = (0, _util.findWrapper)(callee) || (0, _util.findWrapper)(referencePath); // generate a unique name and replace existing references with it

    const id = caller.scope.generateUidIdentifier('arg');
    const param = (0, _util.getParamNode)(t, referencePath, id);
    referencePath.replaceWith(id);
    _referencePath = referencePath, (0, _util.markPlaceholder)(_referencePath);
    _callee = callee, (0, _util.markPlaceholder)(_callee);

    if (wrapper) {
      // we've already wrapped this expression so simply add
      // the above id to the existing wrapper's parameter list
      wrapper.node.params.push(param);
      return;
    } // track this as a location where parameters may need to be hoisted


    hoistTargets.push(caller); // make sure tail paths are kept inside the wrapper
    // (i.e. trailing member expressions like `foo(_).name`)

    const tail = (0, _util.findTopmostLink)(caller); // create an arrow function that wraps and returns the expression
    // generating an arrow maintains lexical `this`

    const fn = t.arrowFunctionExpression([param], t.blockStatement([t.returnStatement(tail.node)])); // replace the expression with the new wrapper that returns it

    tail.replaceWith(fn); // mark the replacement so we can tell that it used to be a placeholder

    _tail = tail, (0, _util.markPlaceholder)(_tail);
  });
  hoistTargets.forEach((_arg) => {
    return (0, _util.hoistArguments)(t, _arg);
  });
};

exports.default = _default;